function Splines_Suavizantes(t,y,CI,S1a,S2a,CD,S1b,S2b,opx,x,opg1,opg2,opg3,opg4,tol_rho,handles)


% Esta funcin busca hallar las ecuaciones que definen los diferentes 
% tramos que componen el spline 
% suavizante y dibujar las grficas de la curva, su primera y segunda 
% derivada, asi como los valores obtenidos al interpolar y su grfica. 
% Creando ficheros que recojan la informacin.
% Splines_Suavizantes(t,y,CI,S1a,S2a,CD,S1b,S2b,opx,x,opg1,opg2,opg3,opg4,tol_rho)
%
% Variables de entrada:
% t:abscisas de la tabla de valores de los nodos
% y:ordenadas de la tabla de valores de los nodos
% CI:condicin de contorno en el extremo izquierdo
% CD:condicin de contorno en  el extremo derecho

% Posibles condiciones de contorno:
% 1- De primer tipo: valores de la primera derivada
% 2- De segundo tipo: valores de la segunda derivada
% 3- De tercer tipo: peridicas de periodo T=b-a
%    S'(a)=S'(b); S''(a)=S''(b). Si CI=3->CD=3
% 4- De cuarto tipo: Tres veces diferenciable en los puntos t2 y/o tm-1
%
% S1a:valor de la primera derivada en el extremo izquierdo
% S2a:valor de la segunda derivada en el extremo izquierdo
% S1b:valor de la primera derivada en el extremo derecho
% S2b:valor de la segunda derivada en el extremo derecho
%
% opx:parmetro que nos indica si debemos calcular o no los valores interpolados
% 's':se calculan; 'n': no se calculan
% x:retculo de puntos donde queremos evaluar la funcin polinmica a trozos

% opg1:parmetro que nos indica se debemos dibujar o no la grfica de los valoes interpolados
% opg2:parmetro que nos indica se debemos dibujar o no la grfica de los splines
% opg3:parmetro que nos indica se debemos dibujar o no la grfica de las primeras derivadas
% opg4:parmetro que nos indica se debemos dibujar o no la grfica de las segundas derivadas
% si opg1=1 entonces dibujar la grfica
% tol_rho vector de tolerancias para los pesos del funcional, ser el que determine el
% grado de suavidad de la curva, es decir, que la curva realiza menos oscilaciones 
%
% EJEMPLO:
% t=[1,3,4,6]
% y=[2,-1,3,1]
% CI=2 ; CD=2
% S1a=0; S1b=0 (no se van a utilizar, da igual el dato)
% S2a=2; S2b=2
% opx='s'
% x=[1:0.2:6]
%
% Splines_Suavizantes([1,3,4,6],[2,-1,3,1],2,0,0,2,0,0,'s',[1:0.2:6],1,1,1,1,[0.001,0.001,0.001,0.001]);

% Nmero de puntos en la tabla, longitud del vector nodos
n=length(t);
pe=length(tol_rho);
% Comprobamos que las entradas son correctas.
% El nmero mnimo de nodos para poder realizar un Spline es 3
if n<3
    uiwait(msgbox('El nmero de puntos de control debe ser mayor que 2', 'Mensaje de error',...
        'error','modal'));
    return;
% Si no se especifican las condiciones en la frontera, se avisar el error
elseif CI==0 | CD==0
    uiwait(msgbox('Falta especificar alguna de las condiciones de frontera', 'Mensaje de error',...
        'error','modal'));
    return;
% Si se marca condiciones peridicas o 3, va intrseco que se tienen que dar en
% los tres exremos
elseif CI==3 & CD~=3
    uiwait(msgbox('Si CI=3 entonces CD debe ser tambin 3', 'Mensaje de error',...
        'error','modal'));
    return;
elseif CD==3 & CI~=3
    uiwait(msgbox('Si CD=3 entonces CI debe ser tambin 3', 'Mensaje de error',...
        'error','modal'));
    return;
elseif CD==3 & y(1)~=y(n)
    uiwait(msgbox('Para que la funcin sea peridica debe valer lo mismo en los extremos', 'Mensaje de error',...
        'error','modal'));
    return;
% Con 3 nodos la condicin CI=4 ya implica CD=4 y nos falta una condicin
elseif n==3 & CI==4 & CD==4
    uiwait(msgbox('Con 3 nodos la condicin CI=4 ya implica CD=4 y nos falta una condicin', 'Mensaje de error',...
        'error','modal'));
    return;
elseif pe<3
     uiwait(msgbox('Se necesitan especificar al menos 3 coeficientes de peso', 'Mensaje de error',...
        'error','modal'));  
end


% Definimos el vector h de distancias entre las abscisas
h=diff(t,1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%  Seleccin de las matrices  %%%%%%%%%%
%%%%%%  coeficiente en funcin     %%%%%%%%%% 
%%%%%%  de las condiciones en la frontera %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Elegimos las matrices coeficiente en funcin de las condiciones de contorno dadas
% Las matrices varan en funcin de las condiciones de la frontera dato
% Elegiremos las matrices mediante un condicional
  
A=condicion_derivadas_A(h,CI,CD);
H=condicion_derivadas_H(h,CI,CD);
V=relacion_EtaLambda_V(h,CI,CD);
U=relacion_EtaLambda_U(h);
J=relacion_ZetaLambda_J(h,CI,CD);
F=relacion_EtaZeta_F(h,CI,CD,S1a,S1b,S2a,S2b);


% Definimos la matriz Auxiliar K1

K1_aux=6*J*V^(-1)*U*A^(-1)*H;
%rho=min(tol_rho,1./sum(abs(K1_aux)))
K1=diag(tol_rho)*K1_aux;

% Definimos la matriz Auxiliar K2
K2=diag(tol_rho)*J*inv(V)*U*inv(A)*F;

% Resolucin del sistema lineal Cz=B
C=eye(n)-1/2*K1;
B=y+K2;
z=C\B;

% Clculo de los valores de la segunda derivada en los nodos eta's

eta=A^(-1)*(6*H*z+F);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           Evaluamos los Splines en x             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


if opx=='s'

    % Encuentra el trozo polinmico a evaluar segn cada entrada de x
    if x(1)==t(1)
        Si(1)=z(1);
        minim=Si; maxim=Si;
    else
        ind=find((x(1)>t)==0);
        ind=ind(1)-1;
        ind=ind(1);
        % Aplica la frmula para calcular S_ind(x)
        tn=(x(1)-t(ind))/h(ind);
        Si(1)=z(ind)*(1-tn)+z(ind+1)*tn-h(ind)^2/6*tn*(1-tn)*((2-tn)*eta(ind)+(1+tn)*eta(ind+1));
        minim=Si; maxim=Si;
    end

    for i=2:length(x)
        ind=find((x(i)>t)==0);
        ind=ind(1)-1;
        ind=ind(1);
        % Aplica la frmula para calcular S_ind(x)
        tn=(x(i)-t(ind))/h(ind);
        Si(i)=z(ind)*(1-tn)+z(ind+1)*tn-h(ind)^2/6*tn*(1-tn)*((2-tn)*eta(ind)+(1+tn)*eta(ind+1));
        if Si(i)<minim
            minim=Si(i);
        elseif Si(i)>maxim
            maxim=Si(i);
        end
    end

    if opg1==1
        figure(1);
        %Dibuja los puntos de control en la grfica
        b1=plot(t,y,'ko','MarkerSize',2,'LineWidth',3);
        hold on;
        %Dibuja los valores interpolados en la grfica
        b2=plot(x,Si,'ro','MarkerSize',3);
        hold on;
        axis([t(1)-0.1*(t(2)-t(1)) t(length(t))+...
            0.1*(t(2)-t(1)) min(min(y),minim)-...
            0.1*(max(y)-min(y)) max(max(y),maxim)+0.1*(max(y)-min(y))]);
        axis equal;
        legend([b1,b2],'Puntos de Control','Valores Aproximados');
    end
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Construimos los polinomios de la interpolacin %
%                 Segmentaria                    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

syms r;

%%%%% Establecemos una variable simblica%%%%%%


for i=1:n-1
    S(i)=z(i)*(1-(r-t(i))/h(i))+z(i+1)*(r-t(i))/h(i)-...
        h(i)^2/6*(r-t(i))/h(i)*(1-(r-t(i))/h(i))*...
        ((2-(r-t(i))/h(i))*eta(i)+(1+(r-t(i))/h(i))*eta(i+1));
end


if opg2==1
    %Dibuja los puntos de control en la grfica
    figure(2);
    b1=plot(t,y,'ko','MarkerSize',2,'LineWidth',3);
    hold on;

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %Dibuja cada uno de los polinomios%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    Ms=0;
    ms=0;
    for i=1:n-1
        b3=ezplot(S(i),[t(i),t(i+1)]);
        auxX=t(i):(t(i+1)-t(i))/10:t(i+1);
        auxY=subs(S(i),auxX); MauxY=max(auxY); mauxY=min(auxY);
        if MauxY>Ms
            Ms=MauxY;
        end
        if mauxY<ms
            ms=mauxY;
        end
    end
    axis([t(1)-0.1*(t(2)-t(1)) t(length(t))+...
        0.1*(t(2)-t(1))  ms-0.1*(Ms-ms)  Ms+0.1*(Ms-ms)]);
    axis equal;
    legend([b1,b3],'Puntos de Control','Splines Cbicos');
    title('Ajuste por Splines Cbicos Suavizantes');
    
    
    
    % dibuja los polinomio splines tambin en la interfaz grfica
    axes(handles.grafica);
   
    
    b1=plot(t,y,'ko','MarkerSize',2,'LineWidth',3);
    hold on;

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %Dibuja cada uno de los polinomios%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    for i=1:n-1
        b3=ezplot(S(i),[t(i),t(i+1)]);
    end
    axis([t(1)-0.1*(t(2)-t(1)) t(length(t))+...
        0.1*(t(2)-t(1))  ms-0.1*(Ms-ms)  Ms+0.1*(Ms-ms)]);
    axis equal;
    legend([b1,b3],'Puntos de Control','Splines Cbicos');
    title('Ajuste por Splines Cbicos Suavizantes');
    hold off
    
end


if opg3==1
    %Dibuja cada una de las primeras derivadas
    figure(3);
    hold on
    Ms=0;
    ms=0;
    
% Diferenciamos el polinomio construido, para obtener el polinomio de la
% primera derivada.
    S1=diff(S,1);
    for i=1:n-1
        b4=ezplot(S1(i),[t(i),t(i+1)]);
        auxX=t(i):(t(i+1)-t(i))/10:t(i+1);
        auxY=subs(S1(i),auxX); MauxY=max(auxY); mauxY=min(auxY);
        if MauxY>Ms
            Ms=MauxY;
        end
        if mauxY<ms
            ms=mauxY;
        end
    end
    axis([t(1)-0.1*(t(2)-t(1)) t(length(t))+...
        0.1*(t(2)-t(1))  ms-0.1*(Ms-ms)  Ms+0.1*(Ms-ms)]);
    legend(b4,'Primera Derivada Splines Cbicos');
    title('Primera Derivada de los Splines Cbicos Suavizantes');
end


if opg4==1
    %Dibuja cada una de las segundas derivadas
    figure(4)
    hold on
    
% Diferenciamos el polinomio construido dos veces, para obtener el polinomio de la
% segunda derivada.
    S2=diff(S,2);
    Msd=subs(S2(1),t(1));
    msd=subs(S2(1),t(1));
    for i=1:n-1
        b5=ezplot(S2(i),[t(i),t(i+1)]);
        aux=subs(S2(i),t(i+1));
        if aux>Msd
            Msd=aux;
        elseif aux<msd
            msd=aux;
        end
    end
    axis([t(1)-0.1*(t(2)-t(1)) t(length(t))+...
        0.1*(t(2)-t(1)) msd-0.1*(Msd-msd)  Msd+0.1*(Msd-msd)]);
    legend(b5,'Segunda Derivada Splines Cbicos');
    title('Segunda Derivada de los Splines Cbicos Suavizantes');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Salida de los polinomio a un fichero
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%  Se abre o crea un archivo y se escribe en l para los resultados de valores aproximados 

if opx=='s'
    fid=fopen('resul_valores_aproximados.txt','w');
    fprintf(fid,'**************************************************************************\n');
    hora=clock;
    fprintf(fid,'Resultado ejecutado el dia %d-%d-%d a las %d : %d : %d\n',...
        hora(3),hora(2),hora(1),hora(4),hora(5),fix(hora(6)));
    fprintf(fid,'**************************************************************************\n\n');
    for i=1:length(x)
        fprintf(fid,'%f %f \n',x(i),Si(i));
    end
    fclose(fid);
end

%  Se abre o crea un archivo y se escribe en l para los resultados de los polinomios del Spline 

fid=fopen('polinomios_splines.txt','w');
fprintf(fid,'**************************************************************************\n');
hora=clock;
fprintf(fid,'Resultado ejecutado el dia %d-%d-%d a las %d : %d : %d\n',...
    hora(3),hora(2),hora(1),hora(4),hora(5),fix(hora(6)));
fprintf(fid,'**************************************************************************\n\n');
signos='----';
for i=1:length(t)-1
    coefi=sym2poly(expand(S(i)));
    for j=1:4
        if coefi(j)>=0
            signos(j)='+';
        end
    end
    fprintf(fid,' %c %f x^3 %c %f x^2 %c %f x %c %f  \n',...
        signos(1),abs(coefi(1)),signos(2),abs(coefi(2)),signos(3),...
        abs(coefi(3)),signos(4),abs(coefi(4)));
end
fprintf(fid,'\n');
fclose(fid);






